open Okcrypto

type 's t = {
  base_url : string;
  app_name : string;
  public_encryption_key : Public_key.t;
  secret_encryption_key : Secret_key.t;
  public_signing_key : Public_key.t;
  secret_signing_key : Secret_key.t;
  session_duration : Core_kernel.Time.Span.t;
  confirmation_duration : Core_kernel.Time.Span.t;
  storage : 's;
}

let default_session_duration = Core_kernel.Time.Span.of_day 14.0

let default_confirmation_duration = Core_kernel.Time.Span.of_min 10.0

let make ?(session_duration = default_session_duration)
    ?(confirmation_duration = default_confirmation_duration) ~base_url ~app_name
    ~public_encryption_key ~secret_encryption_key ~public_signing_key
    ~secret_signing_key storage =
  {
    base_url;
    app_name;
    public_encryption_key;
    secret_encryption_key;
    public_signing_key;
    secret_signing_key;
    session_duration;
    confirmation_duration;
    storage;
  }

let get_base_url { base_url; _ } = base_url

let get_app_name { app_name; _ } = app_name

let get_public_encryption_key { public_encryption_key; _ } =
  public_encryption_key

let get_secret_encryption_key { secret_encryption_key; _ } =
  secret_encryption_key

let get_public_signing_key { public_signing_key; _ } = public_signing_key

let get_secret_signing_key { secret_signing_key; _ } = secret_signing_key

let get_session_duration { session_duration; _ } = session_duration

let get_confirmation_duration { confirmation_duration; _ } =
  confirmation_duration

let get_storage { storage; _ } = storage
