open Base
open Oksql
open Okcrypto

type t = {
  encryption_key : Public_key.t;
  signing_key : Public_key.t;
  name : string;
}
[@@deriving equal]

module App_from_sql = struct
  type application = t

  type t = application

  let of_sql_row row =
    let open Sqlite3.Data in
    match[@warning "-4"] row with
    | [|
     ("encryption_key", TEXT ek); ("signing_key", TEXT sk); ("name", TEXT name);
    |] ->
        let encryption_key = Public_key.of_base64_exn ek in
        let signing_key = Public_key.of_base64_exn sk in
        { encryption_key; signing_key; name }
    | _ -> failwith "Application.of_sql_row: invalid data"
end

let get_encryption_key { encryption_key; _ } = encryption_key

let get_signing_key { signing_key; _ } = signing_key

let get_name { name; _ } = name

let q_find_all =
  {|
SELECT encryption_key, signing_key, name
FROM applications;
|}

let q_insert_app =
  {|
INSERT INTO applications
(encryption_key, signing_key, name)
VALUES
(:encryption_key, :signing_key, :name);
|}

let q_remove_app =
  {|
DELETE FROM applications
WHERE encryption_key = :encryption_key;
|}

let q_find_by_encryption_key =
  {|
SELECT encryption_key, signing_key, name
FROM applications
WHERE encryption_key = :encryption_key;
|}

let find_all db = exec_fetch (module App_from_sql) db q_find_all

let create db ~encryption_key ~signing_key ~name =
  let open Sqlite3.Data in
  exec_only db q_insert_app
    ~params:
      [
        (":encryption_key", TEXT (Public_key.to_base64 encryption_key));
        (":signing_key", TEXT (Public_key.to_base64 signing_key));
        (":name", TEXT name);
      ];
  { encryption_key; signing_key; name }

let remove db ~encryption_key =
  exec_only db q_remove_app
    ~params:
      [
        ( ":encryption_key",
          Sqlite3.Data.TEXT (Public_key.to_base64 encryption_key) );
      ]

let find_by_encryption_key db ~encryption_key =
  let results =
    exec_fetch
      (module App_from_sql)
      db q_find_by_encryption_key
      ~params:
        [
          ( ":encryption_key",
            Sqlite3.Data.TEXT (Public_key.to_base64 encryption_key) );
        ]
  in
  match results with
  | [| app |] -> Some app
  | [||] -> None
  | _ ->
      failwith
        "Application.find_by_encryption_key: more than one application is found"
