(** Application entry in database *)
type t

(** Returns application's public encryption key *)
val get_encryption_key : t -> Okcrypto.Public_key.t

(** Returns application's public signing key *)
val get_signing_key : t -> Okcrypto.Public_key.t

(** Returns application's display name *)
val get_name : t -> string

(** Compares two entries structurally *)
val equal : t -> t -> bool

(** Creates new application entry in the database *)
val create :
  Sqlite3.db ->
  encryption_key:Okcrypto.Public_key.t ->
  signing_key:Okcrypto.Public_key.t ->
  name:string ->
  t

(** Removes application from the database *)
val remove : Sqlite3.db -> encryption_key:Okcrypto.Public_key.t -> unit

(** Retrieves all applications from the database *)
val find_all : Sqlite3.db -> t array

(** Retrieve one application by its encryption key *)
val find_by_encryption_key :
  Sqlite3.db -> encryption_key:Okcrypto.Public_key.t -> t option
