(** User entry in database *)
type t

(** Returns user ID *)
val get_id : t -> string

(** Returns user's password hash *)
val get_password_hash : t -> string

(** Returns user's public key *)
val get_public_key : t -> Okcrypto.Public_key.t

(** Returns user's secret key *)
val get_secret_key : t -> Okcrypto.Secret_key.t

(** Returns user name *)
val get_name : t -> string

(** Compares two entries structurally *)
val equal : t -> t -> bool

(** Creates new user, generates its public and secret keys and persists in database *)
val create : Sqlite3.db -> id:string -> password:string -> name:string -> t

(** Removes user under given ID. Does nothing if user does not exist.*)
val remove : Sqlite3.db -> id:string -> unit

(** Retrieves all users from database *)
val find_all : Sqlite3.db -> t array

(** Retrieves one user by its ID *)
val find_by_id : Sqlite3.db -> id:string -> t option

(** Changes user's password. Does nothing if user does not exist. *)
val change_password : Sqlite3.db -> id:string -> new_password:string -> unit
