open Alcotest
open Base
open Oksql
open Okauth_server
open Okcrypto
open Utils

let test_should_create_app =
  test_case "should insert app into the database" `Quick (fun () ->
      with_okauth_db_in_mem (fun db ->
          let encryption_key, _ = generate_keypair Key_purpose.Encryption in
          let signing_key, _ = generate_keypair Key_purpose.Signing in
          let name = "Test App Name" in
          let returned_app =
            Application.create db ~encryption_key ~signing_key ~name
          in
          let app_in_db =
            Option.value_exn
              (Application.find_by_encryption_key db ~encryption_key)
          in

          (check bool) "inserted app doesn't equal the returned one" true
            (Application.equal returned_app app_in_db);
          (check string) "names don't match" name
            (Application.get_name app_in_db);
          (check bool) "encryption keys don't match" true
            (Public_key.equal encryption_key
               (Application.get_encryption_key app_in_db));
          (check bool) "signing keys don't match" true
            (Public_key.equal signing_key
               (Application.get_signing_key app_in_db))))

let test_should_remove_app =
  test_case "should remove app from the database" `Quick (fun () ->
      with_okauth_db_in_mem (fun db ->
          exec_only db
            "INSERT INTO applications (encryption_key, name, signing_key) \
             VALUES ('dGVzdF9waw', 'Test App Name', 'dGVzdF9zaw')";
          let apps_before = Application.find_all db in
          (check int) "test app has not been inserted" 1
            (Array.length apps_before);

          Application.remove db ~encryption_key:(Public_key.of_string "test_pk");
          let apps_after = Application.find_all db in
          (check int) "test app has not been removed" 0
            (Array.length apps_after)))

let cases = [ test_should_create_app; test_should_remove_app ]
