open Core
open Okauth_server
open Oksql

let with_db_in_mem fn = with_db ":memory:" fn

let with_tmp_file fn =
  let tmp_file_path, _ = Unix.mkstemp "okauth_test" in
  Exn.protect
    ~f:(fun () -> fn tmp_file_path)
    ~finally:(fun () -> Unix.remove tmp_file_path)

let with_okauth_db_in_mem fn =
  with_db_in_mem (fun db ->
      Migration.apply db ~migrations:Schema.migrations;
      fn db)

let has_table db table_name =
  let results =
    exec_fetch
      (module From_sql.Int64)
      db
      "SELECT count(*) = 1 FROM sqlite_master WHERE type = 'table' AND name = \
       :name;"
      ~params:[ (":name", Sqlite3.Data.TEXT table_name) ]
  in
  Int64.equal results.(0) 1L

module MockTerminal = struct
  module type Params = sig
    val stdout : string ref

    val password : string

    val confirmation : bool
  end

  module Make (P : Params) : Terminal.S = struct
    let read_password () = P.password

    let await_confirmation str =
      print_string str;
      P.confirmation

    let print_string str =
      let new_stdout = !P.stdout ^ str in
      P.stdout := new_stdout

    let print_endline str = print_string (str ^ "\n")
  end
end
